﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

using DevExpress.XtraEditors;
using DevExpress.XtraRichEdit;
using DevExpress.XtraRichEdit.API.Native;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 줄 문자열
        /// </summary>
        private const string LINE_STRING = "------------------------------------------------------------------------------------------------------------------------";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.richEditControl.ActiveViewType = RichEditViewType.Simple;

            this.richEditControl.Options.HorizontalScrollbar.Visibility = RichEditScrollbarVisibility.Hidden;
            this.richEditControl.Options.VerticalScrollbar.Visibility   = RichEditScrollbarVisibility.Auto;

            this.richEditControl.Document.DefaultCharacterProperties.FontName  = "나눔고딕코딩";
            this.richEditControl.Document.DefaultCharacterProperties.FontSize  = 12f;
            this.richEditControl.Document.DefaultCharacterProperties.ForeColor = Color.Black;

            this.richEditControl.Document.ParagraphStyles[0].FontName  = "나눔고딕코딩";
            this.richEditControl.Document.ParagraphStyles[0].FontSize  = 12f; 
            this.richEditControl.Document.ParagraphStyles[0].ForeColor = Color.Black;

            this.richEditControl.LoadDocument("sample.rtf");

            this.parseButton.Click += parseButton_Click;
        }

        #endregion


        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 분석 버튼 클릭시 처리하기 - parseButton_Click(sender, e)

        /// <summary>
        /// 분석 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void parseButton_Click(object sender, EventArgs e)
        {
            Document document = this.richEditControl.Document;

            StringBuilder stringBuilder = new StringBuilder();

            Dictionary<Table, string> tableDictionary = new Dictionary<Table, string>();

            foreach(Paragraph paragraph in document.Paragraphs)
            {
                ReadOnlyDocumentImageCollection imageCollection = document.Images.Get(paragraph.Range);

                if(imageCollection == null || imageCollection.Count == 0)
                {
                    Table table = GetTable(paragraph.Range);

                    if(table != null)
                    {
                        if(tableDictionary.ContainsKey(table))
                        {
                            continue;
                        }
                        else
                        {
                            tableDictionary.Add(table, null);

                            stringBuilder.AppendLine($"테이블 : {table.Rows.Count}×{table.Rows[0].Cells.Count}");

                            ParagraphCollection tableParagraphCollection = document.GetParagraphs(table.Range);

                            foreach(Paragraph tableParagraph in tableParagraphCollection)
                            {
                                stringBuilder.AppendLine($"    문단 : {document.GetText(tableParagraph.Range)}");
                            }
                        }
                    }
                    else
                    {
                        stringBuilder.AppendLine($"문단 : {document.GetText(paragraph.Range)}");
                    }
                }
                else
                {
                    DocumentImage documentImage = imageCollection[0];

                    stringBuilder.AppendLine($"이미지 : {documentImage.Size}");
                }
            }

            this.memoEdit.Text = stringBuilder.ToString();

            document.BeginUpdate();

            foreach(KeyValuePair<Table, string> keyValuePair in tableDictionary)
            {
                Table table = keyValuePair.Key;

                ParagraphCollection tableParagraphCollection = document.GetParagraphs(table.Range);

                Table newTable = document.Tables.Create(document.Range.End, 1, 1);

                for(int i = tableParagraphCollection.Count - 1; i > -1; i--)
                {
                    Paragraph tableParagraph = tableParagraphCollection[i];

                    document.InsertText(newTable.Cell(0, 0).Range.Start, document.GetText(tableParagraph.Range) + (i < tableParagraphCollection.Count - 1 ? "\r\n" : ""));
                }

                document.InsertParagraph(document.Range.End);
            }

            document.EndUpdate();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 테이블 구하기 - GetTable(range)

        /// <summary>
        /// 테이블 구하기
        /// </summary>
        /// <param name="range">문서 범위</param>
        /// <returns>테이블</returns>
        private Table GetTable(DocumentRange range)
        {
            Document document = this.richEditControl.Document;

            foreach(Table table in document.Tables)
            {
                if(table.Range.Contains(range.Start))
                {
                    return table;
                }
            }

            return null;
        }

        #endregion
    }
}